<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	if (!defined('_PS_VERSION_'))
		exit;
			
	require_once( _PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkAPI.php");
	require_once( _PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
	
	/*
	 * exécution des travaux
	 *
	 */
	class PolarisLinkAPI_pullchangesModuleFrontController extends PolarisLinkAPI
	{
		/*
			Renvoi vrai si le client est identifié comme marketplace
		*/
		protected function canExportCustomer($customer)
		{
			if (!$this->Link->OPT_ALLOW_E_CUSTOMERS)
				return false;

			// On empêche les invités
			if (!$this->Link->OPT_ALLOW_E_G_CUSTOMERS && $customer->is_guest)
				return false;				

			if (!$this->Link->OPT_ALLOW_E_MK_CUSTOMERS)
			{
				// On empêche les marketplaces en se basant sur le domaine du mail
				$parts = explode("@", $customer->email);
				if (preg_match("/marketplace.amazon(.*)|(.*)spartoo(.*)/i", 
						strtolower(trim($parts[1]))))
					return false;
			}

			// Sinon, c'est que c'est bon !
			return true;
		}

		protected function writeXmlElement($elementName, $value, $attrs = null)
		{
			$attrStr = "";
			if (!empty($attrs))
				foreach ($attrs as $n => $v)
					$attrStr .= " ".$n."=\"".htmlspecialchars($v, ENT_XML1, 'UTF-8')."\"";
			
			return "<".$elementName.$attrStr.">".htmlspecialchars($value, ENT_XML1, 'UTF-8')."</".$elementName.">";
		}
		
		protected function getCustomersChanges(&$tagReprise)
		{
			try
			{
				if (!$this->Link->OPT_ALLOW_E_CUSTOMERS)
					return "";

				$sinceDate = strtotime(($tagReprise["c"]));
				$tagReprise["c"] = date('Y-m-d H:i:s');	// On aura fini à ce stade !
				
				$genders = array();
				foreach (Gender::getGenders() as $gender)
					$genders[$gender->id] = $gender;

				$customers = array();				

				// requête pour avoir les id des clients ayant changés
				foreach (Db::getInstance()->executeS(sprintf("SELECT id_customer FROM "._DB_PREFIX_."customer WHERE deleted = 0 AND date_upd > '%s'", date("Y-m-d H:i:s", $sinceDate)), true, false) as $row)
					$customers[$row["id_customer"]] = $row["id_customer"];
					
				// requête pour avoir les id des clients ayant changés
				foreach (Db::getInstance()->executeS(sprintf("SELECT c.id_customer FROM "._DB_PREFIX_."address AS a LEFT JOIN "._DB_PREFIX_."customer AS c ON (c.id_customer = a.id_customer) WHERE c.deleted = 0 AND a.id_customer > 0 AND c.id_customer IS NOT NULL AND a.date_upd > '%s'", date("Y-m-d H:i:s", $sinceDate)), true, false) as $row)
					$customers[$row["id_customer"]] = $row["id_customer"];

				foreach ($customers as $customer_id)
				{
					try
					{
						$customer = new Customer($customer_id);

						if ($this->canExportCustomer($customer))
						{
							echo "<customer>";						
							echo "<mails>";
							echo $this->writeXmlElement("mail", $customer->email);
							$altmails = array();
							foreach (Db::getInstance()->executeS(sprintf("SELECT DISTINCT CAST(AES_DECRYPT(UNHEX(mail), sha1('%s')) AS char) AS mail FROM "._DB_PREFIX_."polarislink_mails WHERE id_customer = %s", sha1(_COOKIE_KEY_), (int) $customer_id)) as $row)
								if ($customer->email != $row["mail"])
									echo $this->writeXmlElement("mail", $row["mail"]);
							echo "</mails>";
							
							if ($customer->id_gender > 0 && isset($genders[$customer->id_gender]))
							{
								echo $this->writeXmlElement("title", $genders[$customer->id_gender]->name);
								echo $this->writeXmlElement("gender", $genders[$customer->id_gender]->type);
							}

							echo $this->writeXmlElement("firstname", mb_strtolower($customer->firstname) == "inconnu" ? "" : $customer->firstname);
							echo $this->writeXmlElement("lastname", mb_strtolower($customer->lastname) == "inconnu" ? "" : $customer->lastname);
							
							if ( $customer->birthday > "1901-01-01" )
								echo $this->writeXmlElement("birthday", $customer->birthday);
							
							echo $this->writeXmlElement("optin", $customer->newsletter);
							echo $this->writeXmlElement("optin3rd", $customer->optin);
							echo $this->writeXmlElement("notes", $customer->note);						
							
							echo "</customer>\n";
						}
					}	
					catch (Exception $ex)
					{
						throw new Exception(sprintf("Erreur à l'exportation du client PS#%s", $customer_id), $ex->getCode(), $ex);
					}
				}
			}	
			catch (Exception $ex)
			{
				throw new Exception("Erreur à l'exportation des clients", $ex->getCode(), $ex);
			}
		}
		
		protected function getOrders(&$tagReprise)
		{
			try
			{
				if (empty($this->Link->OrderExportStateID) || count($this->Link->OrderExportStateID) <= 0 || !$this->Link->OPT_ALLOW_E_SALES)
					return;
				
				// On évite de rappatrier toutes les ventes !!
				$orderBarrier = $this->module->getOrderBarrier();				

				$currencyConversion = $this->Link->OPT_ALLOW_E_SALES_OTHER_CURRENCIES;

				// Chargement des taux de conversion !
				$currencies = [];
				$linkCurrency = null;
				if ($currencyConversion)
				{
					foreach (Currency::getCurrencies(true) as $c)
						$currencies[$c->id] = $c;

					if (!isset($currencies[$this->Link->CurrencyID]))
						throw new Exception(sprintf("Impossible de trouver la monnaie de la passerelle (#%s)", $this->Link->CurrencyID));

					$linkCurrency = $currencies[$this->Link->CurrencyID];
				}

				$reexport_orders = $this->module->GetReexportedOrders();
				if (count($reexport_orders) > 0)
					$this->module->SetOrderRexported();

				$query = sprintf("SELECT DISTINCT h.id_order_history, h.id_order FROM "._DB_PREFIX_."order_history h LEFT JOIN "._DB_PREFIX_."orders o ON (o.id_order = h.id_order) WHERE (h.id_order_history > %d AND h.id_order >= '%s' AND h.id_order_state IN (%s)) OR (h.id_order IN (%s)) ORDER BY h.id_order_history ASC", 
								isset($tagReprise["o"]) ? $tagReprise["o"] : 0, 
								$orderBarrier,
								join(', ', $this->Link->OrderExportStateID),
								count($reexport_orders) > 0 ? join(', ', $reexport_orders) : "0"
							);

				$results = Db::getInstance()->executeS($query, true, false);
				if ($results === false)
					throw new PrestaShopDatabaseException($this->DB->getMsgError());

				foreach ($results as $row)
				{
					try
					{
						$order = new Order($row["id_order"]);
						if (Validate::isLoadedObject($order))
						{
							$cv_rate = 1;
							$currency = null;
							if ($order->id_currency != $this->Link->CurrencyID)
							{
								if (!$currencyConversion)
									continue;						

								// Cherchons la devise de la commande
								if (!isset($currencies[$order->id_currency]))
								{
									// Non trouvée : la commande ne sort pas 
									continue;
								}

								$currency = $currencies[$order->id_currency];

								// Monnaie de la commande -> monnaie du site -> Monnaie gérée de la passerelle 
								$cv_rate = $linkCurrency->conversion_rate / $order->conversion_rate;
							}
							
							$customer = new Customer($order->id_customer);
							echo "<order id=\"".$order->id."\">";
							
							echo $this->writeXmlElement("reference", $order->reference);
							$time = strtotime($order->date_add);
							echo $this->writeXmlElement("date", date("Y-m-d H:i:s", $time - date("Z", $time)));

							echo $this->writeXmlElement("payment", $order->module);

							if ($this->Link->OPT_ALLOW_E_CUSTOMERS && 									
									Validate::isLoadedObject($customer) &&
									$this->canExportCustomer($customer))
								echo $this->writeXmlElement("mail", $customer->email);
																			
							$discounts = $order->total_discounts_tax_excl * $cv_rate;
							$discounts_wt = $order->total_discounts_tax_incl * $cv_rate;
							
							$vouchers = array();
							foreach ($order->getCartRules() as $discount)
							{
								$voucher = new CartRule($discount["id_cart_rule"]);
								if (Validate::isLoadedObject($voucher))
									if (!empty($voucher->code))
									{
										$vouchers[] = array("code" => $voucher->code, "amount" => $discount["value"] * $cv_rate);
										$discounts_wt -= ($discount["value"] * $cv_rate);
										$discounts -= ($discount["value_tax_excl"] * $cv_rate);
									}
							}
							
							if (count($vouchers) > 0)
							{
								echo "<vouchers>";
								foreach ($vouchers as $voucher)
									echo $this->writeXmlElement("voucher", sprintf("%.02f", $voucher["amount"] * $cv_rate), array("code" => $voucher["code"]));
								echo "</vouchers>";
							}

							$payments = OrderPayment::getByOrderReference($order->reference);
							if ($payments && count($payments))
							{
								echo "<payments>";
								foreach ($payments as $payment)
								{
									$args = [
										"type" => $this->Link->OPT_PAYMENT_MODULE ? $order->module : $payment->payment_method,
										"date" => date("Y-m-d H:i:s", strtotime($payment->date_add) - date("Z", $time)),
										"tid" => $payment->transaction_id,
									];

									if ($order->id_currency != $this->Link->CurrencyID)
									{
										$args["type"] .= ("/".strtoupper($currency->iso_code));
										$args["devise"] = sprintf("%.02f", $payment->amount);
									}
									
									echo $this->writeXmlElement("payment", sprintf("%.02f", $payment->amount * $cv_rate), $args);
								}
								echo "</payments>";
							}
							
							echo $this->writeXmlElement("total_discounts_tax_incl", sprintf("%0.2f", $discounts_wt));
							echo $this->writeXmlElement("total_discounts_tax_excl", sprintf("%0.2f", $discounts));
							echo $this->writeXmlElement("total_products_tax_incl", sprintf("%0.2f", $order->total_products_wt * $cv_rate));
							echo $this->writeXmlElement("total_products_tax_excl", sprintf("%0.2f", $order->total_products * $cv_rate));
							echo $this->writeXmlElement("total_shipping_tax_incl", sprintf("%0.2f", $order->total_shipping_tax_incl * $cv_rate));
							echo $this->writeXmlElement("total_shipping_tax_excl", sprintf("%0.2f", $order->total_shipping_tax_excl * $cv_rate));
							echo $this->writeXmlElement("total_shipping_tax_rate", sprintf("%0.2f", $order->carrier_tax_rate * $cv_rate));
							echo $this->writeXmlElement("total_wrapping_tax_incl", sprintf("%0.2f", $order->total_wrapping_tax_incl * $cv_rate));
							echo $this->writeXmlElement("total_wrapping_tax_excl", sprintf("%0.2f", $order->total_wrapping_tax_excl * $cv_rate));					
							echo $this->writeXmlElement("total_wrapping_tax_rate", sprintf("%0.2f", $order->carrier_tax_rate * $cv_rate));
							
							echo $this->writeXmlElement("total_tax_incl", sprintf("%0.2f", 
									($order->total_products_wt * $cv_rate) + 
									($order->total_shipping_tax_incl * $cv_rate) + 
									($order->total_wrapping_tax_incl * $cv_rate)));
							echo $this->writeXmlElement("total_paid", sprintf("%0.2f", $order->total_paid * $cv_rate));
							
							echo "<details>";
							foreach ($order->getProducts() as $detail)
							{
								$sku = $detail["product_supplier_reference"];
								echo "<product sku=\"".$sku."\" ean13=\"".$detail["product_ean13"]."\">";
								
								if ((int) $detail["id_warehouse"] > 0)
								{
									$warehouse = new Warehouse($detail["id_warehouse"]);
									if (Validate::isLoadedObject($warehouse))
										echo $this->writeXmlElement("warehouse", $warehouse->reference);
								}
								
								echo $this->writeXmlElement("qty", $detail["product_quantity"]);
								echo $this->writeXmlElement("tax_rate", $detail["tax_rate"]);
								echo $this->writeXmlElement("product_price_tax_excl", sprintf("%0.2f", $detail["original_product_price"] * $cv_rate));
								echo $this->writeXmlElement("product_price_tax_incl", sprintf("%0.2f", 
									$detail["original_product_price"] * $cv_rate * (1 + ($detail["tax_rate"] / 100.0))));
								
								echo $this->writeXmlElement("unit_price_tax_excl", sprintf("%0.2f", $detail["unit_price_tax_excl"] * $cv_rate));
								echo $this->writeXmlElement("unit_price_tax_incl", sprintf("%0.2f", $detail["unit_price_tax_incl"] * $cv_rate));
								
								echo $this->writeXmlElement("total_price_tax_excl", sprintf("%0.2f", $detail["total_price_tax_excl"] * $cv_rate));
								echo $this->writeXmlElement("total_price_tax_incl", sprintf("%0.2f", $detail["total_price_tax_incl"] * $cv_rate));
								
								echo $this->writeXmlElement("reduction_percent", sprintf("%0.2f", $detail["reduction_percent"]));
								echo $this->writeXmlElement("reduction_amount_tax_excl", sprintf("%0.2f", $detail["reduction_amount_tax_excl"] * $cv_rate));
								echo $this->writeXmlElement("reduction_amount_tax_incl",sprintf("%0.2f",  $detail["reduction_amount_tax_incl"] * $cv_rate));
								
								echo "</product>";
							}
							echo "</details>";
							
							echo "</order>\n";

							$tagReprise["o"] = $row["id_order_history"];							
						}
					}	
					catch (Exception $ex)
					{
						throw new Exception(sprintf("Erreur à l'exportation de la commande PS#%s", $row["id_order"]), $ex->getCode(), $ex);
					}
				}				
			}	
			catch (Exception $ex)
			{
				throw new Exception("Erreur à l'exportation des commandes", $ex->getCode(), $ex);
			}
		}
		
		protected function runAPI()
		{
			// Surtout plus de cache !!!
			Db::getInstance()->disableCache();

			$tagReprise = isset($_REQUEST['since']) ? trim($_REQUEST['since']) : null;
			if (empty($tagReprise))
				throw new Exception("paramètre «since» non défini");			
			
			// tagReprise est un JSON en base64
			$since = json_decode(base64_decode($tagReprise), true);

			if (!$since)
			{
				$since = [
					"c" => $tagReprise,
					"o" => 0
				];
			}
			
			// Demande de renvoi total
			if ($this->Link->ReExportCustomers > 0) 
				$since["c"] = "2000-01-01";
			if ($this->Link->ReExportSales > 0)	// Demande de renvoi total
				$since["o"] = 0;

			ob_start();
			try
			{				
				$this->getCustomersChanges($since);
				$this->getOrders($since);								
			}
			catch (Exception $ex)
			{
				ob_end_clean();
				throw $ex;
			}

			$buffer = ob_get_contents();
			ob_end_clean();

			header("Content-Type: application/xml; charset=utf-8");
			echo '<?xml version="1.0" encoding="UTF-8"?>'."\n";
			echo "<changes date=\"".htmlspecialchars(base64_encode(json_encode($since)))."\">\n";			
			echo $buffer;
			echo "</changes>\n";

			// Tout à été renvoyé
			if ($this->Link->ReExportCustomers > 0)
			{		
				$lnk = $this->Link->reloadConfig();
				$lnk->ReExportCustomers = 0;
				$lnk->save();
			}

			if ($this->Link->ReExportSales > 0)
			{
				$lnk = $this->Link->reloadConfig();
				$lnk->ReExportSales = 0;
				$lnk->save();
			}

			return true;
		}
	}
 